<?php
/*
 Plugin Name: PopupAlly Pro
 Plugin URI: http://ambitionally.com/popupally-pro/
 Description: Want even more freedom and customization for your popups? PopupAlly Pro adds two new ways to trigger a popup (scroll to open, and click trigger) and a myriad of style customizations (by popular demand, fonts are now customizable, too)! And for developers, you can enjoy absolute control over every aspect of the PopupAlly's appearance in the Advanced Editing Mode.
 Version: 1.5.8
 Author: Nathalie Lussier Media Inc.
 Author URI: http://nathalielussier.com/
 */

if (!class_exists('PopupAllyPro')) {
	class PopupAllyPro {
		/// CONSTANTS
		const VERSION = '1.5.8';

		const LITE_SETTING_KEY_DISPLAY = '_popupally_setting_general';
		const LITE_SETTING_KEY_STYLE = '_popupally_setting_style';
		const LITE_SETTING_KEY_ADVANCED = '_popupally_setting_advanced';

		const SETTING_KEY_ALL = '_popupally_pro_setting';
		const SETTING_KEY_ENABLED = '_popupally_pro_setting_enabled';
		const SETTING_KEY_CODE = '_popupally_pro_setting_code';
		const SETTING_KEY_DISPLAY = '_popupally_pro_setting_general';
		const SETTING_KEY_STYLE = '_popupally_pro_setting_style';
		const SETTING_KEY_COPY_DELETE = '_popupally_pro_setting_copy_delete';	# not used in database
		const SETTING_KEY_ADVANCED = '_popupally_pro_setting_advanced';
		const SETTING_KEY_LICENSE = '_popupally_pro_setting_license';
		const SETTING_KEY_NUM_STYLE_SAVED = '_popupally_pro_setting_num_style_saved';

		const HELP_URL = 'http://access.nathalielussier.com/popupally-pro/video-tutorials/';

		const SCRIPT_FOLDER = 'popupally-pro-scripts';

		// CACHE
		const CACHE_PERIOD = 86400;

		const TEMPLATE_DIRECTORY = 'template';
		private static $template_folders = array('default', 'clean', 'full-width', 'choice', 'big-logo', 'free', 'point', 'ebook', 'video', 'image', 'three-choice');

		public static $PLUGIN_URI = '';
		public static $popupally_pro_enabled = false;
		public static $available_templates = array();
		public static $available_fonts = null;
		public static $available_fonts_template = null;
		public static $available_font_weights = null;
		public static $available_font_weights_template = null;
		public static $available_alignments = null;
		public static $available_alignments_template = null;

		private static $default_popup_display_settings = null;
		private static $default_popup_style_simple_settings = null;
		private static $default_display_settings = null;
		private static $default_style_settings = null;
		private static $default_advanced_settings = null;
		private static $default_license_settings = null;

		// used for parameter parsing
		private static $config_display_settings = array('cookie-duration', 'priority', 'fade-in');
		private static $config_style_settings = array('popup-selector', 'popup-class', 'cookie-name', 'close-trigger', 'close-iframe-selector');

		private static $display_setting_template_display_types = array('timed', 'exit-intent', 'scroll', 'click', 'embedded');
		private static $display_setting_template_replace = array('timed-popup-delay', 'scroll-percent', 'scroll-trigger', 'priority', 'open-trigger', 'cookie-duration', 'utm-source', 'fade-in');
		private static $display_setting_template_checked_replace = array('timed', 'enable-exit-intent-popup', 'scroll', 'click', 'enable-embedded', 'show-all', 'disable-mobile');
		private static $display_setting_template_embedded_locations = array('none', 'top-page', 'top-page-follow', 'post-start', 'post-end', 'page-end', 'page-end-follow', 'shortcode', 'widget');
		private static $style_settings_template_replace = array('advanced', 'name', 'html', 'html-embedded', 'css', 'css-top-margin', 'popup-selector', 'popup-class', 'close-trigger', 'close-iframe-selector', 'cookie-name', 'sign-up-form-action', 'signup-form');
		private static $style_setting_template_signup_html = array('form', 'name', 'lname', 'email');

		public static function init() {
			self::$PLUGIN_URI = plugin_dir_url(__FILE__);
			self::check_license_status();
			self::add_actions();
			self::add_filters();
			self::load_templates();

			register_activation_hook(__FILE__, array(__CLASS__, 'do_activation_actions'));
			register_deactivation_hook(__FILE__, array(__CLASS__, 'do_deactivation_actions'));
		}

		public static function do_activation_actions() {
			delete_transient(self::SETTING_KEY_ALL);
			delete_option(self::SETTING_KEY_ALL);
			delete_transient(self::SETTING_KEY_CODE);
			delete_option(self::SETTING_KEY_CODE);
			delete_transient(self::SETTING_KEY_DISPLAY);
			delete_transient(self::SETTING_KEY_STYLE);
			delete_transient(self::SETTING_KEY_ADVANCED);
			delete_transient(self::SETTING_KEY_LICENSE);
			delete_transient(self::SETTING_KEY_NUM_STYLE_SAVED);
			delete_transient(self::SETTING_KEY_ENABLED);

			PopupAllyProUpdater::do_activation_actions();

			$display = get_option(self::LITE_SETTING_KEY_DISPLAY);
			if (false === $display) {
				$display = self::get_default_display_setting();
			} else {
				foreach($display as $id => $setting) {
					$display[$id] = wp_parse_args($setting, self::get_default_popup_display_setting($id));
				}
			}
			if (add_option(self::SETTING_KEY_DISPLAY, $display)) {
				set_transient(self::SETTING_KEY_DISPLAY, $display, self::CACHE_PERIOD);
			}
			$lite_style = get_option(self::LITE_SETTING_KEY_STYLE);
			if (false === $lite_style) {
				$style = self::get_default_style_setting();
			} else {
				$style = array();
				foreach($lite_style as $id => $setting) {
					$style[$id] = wp_parse_args($setting, self::get_default_popup_style_setting($id));
					if (isset($setting['text-color'])) {
						$style[$id]['headline-color'] = $style[$id]['sales-text-color'] = $style[$id]['input-box-color'] = $style[$id]['privacy-text-color'] = $setting['text-color'];
					}
				}
				foreach (self::$available_templates as $template) {
					$template->do_activation_actions($style, $lite_style);
				}
			}
			if (add_option(self::SETTING_KEY_STYLE, $style)) {
				set_transient(self::SETTING_KEY_STYLE, $style, self::CACHE_PERIOD);
			}
			$advanced = get_option(self::LITE_SETTING_KEY_ADVANCED);
			if (false === $advanced) {
				$advanced = self::get_advanced_settings();
			}
			// always reset the max number of page/post to 500, or the settings page might not load due to time out
			$advanced['max-page'] = '500';
			$advanced['max-post'] = '500';
			update_option(self::SETTING_KEY_ADVANCED, $advanced);
			set_transient(self::SETTING_KEY_ADVANCED, $advanced, self::CACHE_PERIOD);

			if (add_option(self::SETTING_KEY_NUM_STYLE_SAVED, 0)) {
				set_transient(self::SETTING_KEY_NUM_STYLE_SAVED, 0, self::CACHE_PERIOD);
			}
		}

		public static function do_deactivation_actions() {
			delete_transient(self::SETTING_KEY_ALL);
			delete_option(self::SETTING_KEY_ALL);
			delete_transient(self::SETTING_KEY_CODE);
			delete_option(self::SETTING_KEY_CODE);
			delete_transient(self::SETTING_KEY_DISPLAY);
			delete_transient(self::SETTING_KEY_STYLE);
			delete_transient(self::SETTING_KEY_ADVANCED);
			delete_transient(self::SETTING_KEY_LICENSE);
			delete_transient(self::SETTING_KEY_NUM_STYLE_SAVED);
			delete_transient(self::SETTING_KEY_ENABLED);

			PopupAllyProUpdater::do_deactivation_actions();
		}

		private static function add_actions() {
			if (is_admin()) {
				add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_administrative_resources'));

				if (self::$popupally_pro_enabled) {
					add_action('add_meta_boxes', array(__CLASS__, 'add_meta_box'));
					add_action('admin_init', array(__CLASS__, 'process_export'));
				}

				// add setting menu
				add_action('admin_menu', array(__CLASS__, 'add_menu_pages'));
				add_action('admin_init', array(__CLASS__, 'register_settings'));
			}

			if (self::$popupally_pro_enabled) {
				add_action('widgets_init', array(__CLASS__, 'register_widgets'));	

				add_action('init', array(__CLASS__, 'add_shortcodes'));
				add_action('wp_enqueue_scripts', array(__CLASS__, 'enqueue_resources'));
				add_action('wp_head', array(__CLASS__, 'add_popup_scripts'), 99);
				add_action('wp_footer', array(__CLASS__, 'add_popup_html'));
			}
		}

		private static function add_filters() {
			if (self::$popupally_pro_enabled) {
				add_filter('the_content', array(__CLASS__, 'add_form_to_content'));
			}
		}

		public static function register_settings() {
			register_setting('popupally_pro_settings', self::SETTING_KEY_ALL, array(__CLASS__, 'sanitize_settings'));
		}

		public static function enqueue_resources() {
			wp_enqueue_script('jquery');

			$display = self::get_display_settings();
			$style = self::get_style_settings();
			$utm_mapping = array();
			foreach($display as $id => $setting) {
				$utm_mapping[$style[$id]['cookie-name']] = $setting['utm-source'];
			}
			wp_enqueue_script('popupally-pro-check-source', plugin_dir_url(__FILE__) . 'resource/frontend/check-source.min.js', false, self::VERSION);
			wp_localize_script('popupally-pro-check-source', 'popupally_pro_check_source_object', array('utm_mapping' => $utm_mapping));

			$advanced = self::get_advanced_settings();
			if ('true' === $advanced['no-inline']) {
				$num_saved = self::get_num_style_saved_settings();
				$to_show = self::get_popup_to_show();
				if (!empty($to_show)) {
					wp_enqueue_script('popupally-pro-action-script', plugin_dir_url(__FILE__) . 'resource/frontend/popup.min.js', false, self::VERSION);
					wp_enqueue_style('popupally-pro-style', content_url(self::SCRIPT_FOLDER) . '/popupally-pro-style.css', false, self::VERSION . '.' . $num_saved);
				}
				$ids = self::get_popup_thank_you_to_show();
				if (!empty($ids)){
					foreach($ids as $id) {
						wp_enqueue_script('popupally-pro-thank-you-script-' . $id, content_url(self::SCRIPT_FOLDER) . '/popupally-pro-thank-you-' . $id . '.js', false, self::VERSION . '.' . $num_saved);
					}
				}
			}
		}

		public static function enqueue_administrative_resources($hook) {
			if (strpos($hook, self::SETTING_KEY_ALL) !== false) {
				wp_enqueue_style('wp-color-picker');
				wp_enqueue_style('popupally-pro-backend', plugin_dir_url(__FILE__) . 'resource/backend/popupally.css', false, self::VERSION);

				wp_enqueue_script('popupally-pro-backend', plugin_dir_url(__FILE__) . 'resource/backend/popupally.js', array('jquery'), self::VERSION);
				wp_enqueue_script('popupally-pro-backend-color-picker', plugin_dir_url(__FILE__) . 'resource/backend/jscolor/jscolor.js', array('jquery'), self::VERSION);

				wp_enqueue_script('popupally-pro-backend-preview', plugin_dir_url(__FILE__) . 'resource/backend/popup-simple-preview.js', array('jquery'), self::VERSION);
				wp_localize_script( 'popupally-pro-backend', 'popupally_pro_data_object',
					array( 'display_setting_variable' => PopupAllyPro::SETTING_KEY_DISPLAY, 'display_default_setting' => self::get_default_popup_display_setting(),
						'style_setting_variable' => PopupAllyPro::SETTING_KEY_STYLE, 'style_default_setting' => self::get_default_popup_style_setting()));
			}
		}

		// <editor-fold defaultstate="collapsed" desc="Templates">
		public static function add_template($template) {
			self::$available_templates[$template->uid] = $template;
		}

		public static function compare_template_order($a, $b)
		{
			if ($a->template_order == $b->template_order) {
				return 0;
			}
			return ($a->template_order < $b->template_order) ? -1 : 1;
		}

		private static function load_templates() {
			$root = dirname(__FILE__) . '/resource/' .self::TEMPLATE_DIRECTORY;
			foreach (self::$template_folders as $folder) {
				$file = $root . '/' . $folder . '/definition.php';
				include_once($file);
			}
			uasort(self::$available_templates, array(__CLASS__, 'compare_template_order'));
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Shortcodes">
		public static function add_shortcodes() {
			add_shortcode( 'embed_popupally_pro', array(__CLASS__, 'shortcode_embed_popupally_pro'));
		}

		public static function shortcode_embed_popupally_pro($atts) {
			extract( shortcode_atts( array(
				'popup_id' => '0',
			), $atts ) );
			$to_show = self::get_popup_to_show();
			if (isset($to_show[$popup_id])) {
				$display = self::get_display_settings();
				if (in_array('embedded', $to_show[$popup_id]) && $display[$popup_id]['embedded-location'] === 'shortcode') {
					$code = self::get_popup_code();
					if (isset($code[$popup_id])) {
						return $code[$popup_id]['embedded_html'];
					}
				}
			}
			return '';
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Widgets">
		public static function register_widgets() {
			register_widget( 'popupally_pro_embedded_widget' );
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Embedded sign up forms">
		public static function add_form_to_content($content) {
			if (!is_singular()) {
				return $content;
			}
			$to_show = self::get_popup_to_show();
			if (!empty($to_show)) {
				$display = self::get_display_settings();
				$code = self::get_popup_code();
				foreach($to_show as $id => $popup_types) {
					if (in_array('embedded', $popup_types)) {
						if ('post-start' === $display[$id]['embedded-location']) {
							$content = $code[$id]['embedded_html'] . $content;
						} elseif ('post-end' === $display[$id]['embedded-location']) {
							$content = $content . $code[$id]['embedded_html'];
						}
					}
				}
			}
			return $content;
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Page meta box">
		public static function add_meta_box($post_type) {
			$post_types = array('post', 'page');     //limit meta box to certain post types
			if ( in_array( $post_type, $post_types )) {
				add_meta_box(
					'popupally-pro-display-settings',
					 'PopupAlly Pro Display Settings',
					array( __CLASS__, 'show_post_display_meta_box_content' ),
					$post_type,
					'side',
					'high'
				);
			}
		}

		public static function show_post_display_meta_box_content($post) {
			$to_show = self::get_popup_to_show($post->ID, true);
			$style = self::get_style_settings();

			include (dirname(__FILE__) . '/resource/backend/post-display.php');
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Settings">
		public static function add_menu_pages() {
			// Add the top-level admin menu
			$capability = 'manage_options';
			$plugin_page = add_menu_page('PopupAlly Pro Settings', 'PopupAlly Pro', $capability, self::SETTING_KEY_ALL, array(__CLASS__, 'show_popupally_pro_settings'), self::$PLUGIN_URI . 'resource/backend/img/popupally-pro-icon.png');

			if (self::$popupally_pro_enabled) {
				add_action('admin_head-'.$plugin_page, array(__CLASS__, 'add_preview_popup_scripts'));
			}
		}

		public static function show_popupally_pro_settings() {
			if (!current_user_can('manage_options')) {
				wp_die('You do not have sufficient permissions to access this page.');
			}
			self::check_php_version('popupally_pro_settings');
			$setting = self::get_selected_settings();
			include (dirname(__FILE__) . '/resource/backend/setting-all.php');
		}

		private static function generate_page_selection_html($template, $id, $type, $setting, $page_template, $pages, $categories, $custom_page_types, $custom_page_templates, $custom_pages) {
			$html = str_replace('{{id}}', $id, $template);
			$html = str_replace('{{type}}', $type, $html);

			$html = str_replace('{{all-pages}}', isset($setting['all-pages'])?'checked':'', $html);
			$html = str_replace('{{front-page}}', isset($setting['front-page'])?'checked':'', $html);

			$html = str_replace('{{page-selection}}', self::generate_checkbox_list($id, $type, 'page', $setting, $page_template, $pages), $html);

			$category_selection = '';
			foreach ($categories as $category) {
				$category_selection .= '<li>';
				$category_selection .= '<input class="' . $type . '-page-' . $id . '" ' . (isset($setting['category-' . $category->cat_ID])?'checked':'') . ' id="' . $type . '-' . $id . '-category-' . $category->cat_ID .
						'" type="checkbox" value="true" name="[' . $id . '][' . $type . '][category-' . $category->cat_ID .
						']"><label for="' . $type . '-' . $id . '-category-' . $category->cat_ID . '">' . esc_attr($category->name) . '</label>';
				$category_selection .= '</li>';
			}
			$html = str_replace('{{category-page-selection}}', $category_selection, $html);

			$custom_page_selection = '';
			foreach($custom_page_types as $custom_page_type) {
				$custom_page_selection .= '<input type="checkbox" ' . (isset($setting['all-'.$custom_page_type->name])?'checked':'') . ' class="' . $type . '-page-' . $id . '" id="' . $type . '-all-' . $custom_page_type->name . '-' . $id .
						'" value="true" name="[' . $id . '][' . $type . '][all-' . $custom_page_type->name . ']"><label for="' . $type .
						'-all-' . $custom_page_type->name . '-' . $id. '">All ' . esc_attr($custom_page_type->label) . '</label>';
				$custom_page_selection .= '<ul>';
				$custom_page_selection .= self::generate_checkbox_list($id, $type, $custom_page_type->name, $setting, $custom_page_templates[$custom_page_type->name], $custom_pages[$custom_page_type->name]);
				$custom_page_selection .= '</ul>';
			}
			$html = str_replace('{{custom-page-selection}}', $custom_page_selection, $html);
			return $html;
		}

		private static function generate_post_selection_html($template, $id, $type, $setting, $post_template, $posts, $categories, $custom_post_types, $custom_post_templates, $custom_posts) {
			$html = str_replace('{{id}}', $id, $template);
			$html = str_replace('{{type}}', $type, $html);

			$html = str_replace('{{all-posts}}', isset($setting['all-posts'])?'checked':'', $html);

			$html = str_replace('{{post-selection}}', self::generate_checkbox_list($id, $type, 'post', $setting, $post_template, $posts), $html);

			$category_selection = '';
			foreach ($categories as $category) {
				$category_selection .= '<li>';
				$category_selection .= '<input class="' . $type . '-post-' . $id . '" ' . (isset($setting['category-post-' . $category->cat_ID])?'checked':'') . ' id="' . $type . '-' . $id .
						'-category-post-' . $category->cat_ID . '" type="checkbox" value="true" name="[' . $id . '][' . $type . '][category-post-' . $category->cat_ID . ']"><label for="' . $type . '-' . $id . '-category-post-' . $category->cat_ID . '">' . esc_attr($category->name) . '</label>';
				$category_selection .= '</li>';
			}
			$html = str_replace('{{category-post-selection}}', $category_selection, $html);

			$custom_post_selection = '';
			foreach($custom_post_types as $custom_post_type) {
				$custom_post_selection .= '<input type="checkbox" ' . (isset($setting['all-'.$custom_post_type->name])?'checked':'') . ' class="' . $type . '-post-' . $id . '" id="' . $type .
						'-all-' . $custom_post_type->name . '-' . $id .'" value="true" name="[' . $id . '][' . $type . '][all-' . $custom_post_type->name . ']' .
						'"><label for="' . $type . '-all-' . $custom_post_type->name . '-' . $id . '">All ' . esc_attr($custom_post_type->label) . '</label>';
				$custom_post_selection .= '<ul>';
				$custom_post_selection .= self::generate_checkbox_list($id, $type, $custom_post_type->name, $setting, $custom_post_templates[$custom_post_type->name], $custom_posts[$custom_post_type->name]);
				$custom_post_selection .= '</ul>';
			}
			$html = str_replace('{{custom-post-selection}}', $custom_post_selection, $html);
			return $html;
		}
		private static function generate_display_type_warning_templates() {
			$result = array();
			foreach(self::$display_setting_template_display_types as $type) {
				$result[$type] = '';
				foreach(self::$available_templates as $template) {
					if ($template->display_type_warning[$type]) {
						$result[$type] .= $template->uid . ',';
					}
				}
				if (strlen($result[$type]) > 0) {
					$result[$type] = 'template-warning-target="' . $result[$type] . '"';
				}
			}
			return $result;
		}
		public static function show_display_settings() {
			$display = self::get_display_settings();
			$style = self::get_style_settings();

			$disable = file_get_contents(dirname(__FILE__) . '/resource/frontend/disable.php');

			$advanced = self::get_advanced_settings();
			if ($advanced['max-page'] < 0) {
				$pages = get_pages();
			} else {
				$pages = get_pages(array('number' => $advanced['max-page']));
			}
			$page_template = self::generate_page_checkbox_template($pages);
			$posts = get_posts(array('posts_per_page' => $advanced['max-post']));
			$post_template = self::generate_post_checkbox_template($posts);
			$categories = get_categories(array('hide_empty' => false));

			$custom_page_types = get_post_types(array('public' => 'true', 'capability_type' => 'page', '_builtin' => false), 'object');
			$custom_page_templates = array();
			$custom_pages = array();
			foreach($custom_page_types as $custom_page_type) {
				if ($advanced['max-page'] < 0) {
					$custom_pages[$custom_page_type->name] = get_pages(array('post_type' => $custom_page_type->name));
				} else {
					$custom_pages[$custom_page_type->name] = get_pages(array('number' => $advanced['max-page'], 'post_type' => $custom_page_type->name));
				}
				$custom_page_templates[$custom_page_type->name] = self::generate_page_checkbox_template($custom_pages[$custom_page_type->name]);
			}

			$custom_post_types = get_post_types(array('public' => 'true', 'capability_type' => 'post', '_builtin' => false), 'object');
			$custom_post_templates = array();
			$custom_posts = array();
			foreach($custom_post_types as $custom_post_type) {
				$custom_posts[$custom_post_type->name] = get_posts(array('posts_per_page' => $advanced['max-post'], 'post_type' => $custom_post_type->name));
				$custom_post_templates[$custom_post_type->name] = self::generate_post_checkbox_template($custom_posts[$custom_post_type->name]);
			}
			$display_type_warning_templates = self::generate_display_type_warning_templates();
			echo '<h3>Display Settings</h3>';

			$display_detail_template = file_get_contents(dirname(__FILE__) . '/resource/backend/setting-display-popup-template.php');

			$host_url = esc_attr(get_bloginfo('url'));

			$page_selection_template = file_get_contents(dirname(__FILE__) . '/resource/backend/setting-display-page-selection-template.php');
			$post_selection_template = file_get_contents(dirname(__FILE__) . '/resource/backend/setting-display-post-selection-template.php');
			foreach ($display as $id => $setting) {
				$display_detail = $display_detail_template;
				foreach(self::$display_setting_template_replace as $replace) {
					$display_detail = str_replace("{{{$replace}}}", esc_attr($setting[$replace]), $display_detail);
				}
				foreach(self::$display_setting_template_checked_replace as $replace) {
					if ($setting[$replace] === 'true') {
						$display_detail = str_replace("{{{$replace}}}", 'checked', $display_detail);
						$display_detail = str_replace("{{{$replace}-false-hide}}", '', $display_detail);
					} else {
						$display_detail = str_replace("{{{$replace}}}", '', $display_detail);
						$display_detail = str_replace("{{{$replace}-false-hide}}", 'style="display:none;"', $display_detail);
					}
				}
				if ($setting['embedded-location'] === 'shortcode') {
					$display_detail = str_replace("{{embedded-location-shortcode-show}}", '', $display_detail);
				} else {
					$display_detail = str_replace("{{embedded-location-shortcode-show}}", 'style="display:none;"', $display_detail);
				}
				$display_detail = str_replace("{{id}}", $id, $display_detail);
				$display_detail = str_replace("{{selected_item_opened}}", $setting['is-open'] === 'true'?'popupally-item-opened':'', $display_detail);
				$display_detail = str_replace("{{selected_item_checked}}", $setting['is-open'] === 'true'?'checked':'', $display_detail);
				$display_detail = str_replace("{{selected_item_false_hide}}", $setting['is-open'] !== 'true'?'style="display:none;"':'', $display_detail);
				$display_detail = str_replace("{{selected_item_true_hide}}", $setting['is-open'] === 'true'?'style="display:none;"':'', $display_detail);
				$display_detail = str_replace("{{name}}", $style[$id]['name'], $display_detail);
				$display_detail = str_replace("{{show-all-false}}", $setting['show-all'] === "true"?"none":"block", $display_detail);
				$display_detail = str_replace("{{show-all-true}}", $setting['show-all'] === "true"?"block":"none", $display_detail);
				$display_detail = str_replace("{{cookie-js}}", esc_attr(str_replace('##cookie_name##', $style[$id]['cookie-name'], $disable)), $display_detail);
				$display_detail = str_replace("{{show-thank-you}}", empty($setting['thank-you']) ? '' : 'checked', $display_detail);
				$display_detail = str_replace("{{show-thank-you-hide}}", empty($setting['thank-you']) ? 'style="display:none;"' : '', $display_detail);
				$display_detail = str_replace("{{host-url}}", $host_url, $display_detail);

				$has_display_option_selected = 'true' === $setting['timed'] || 'true' === $setting['enable-exit-intent-popup'] || 'true' === $setting['scroll'] ||
						'true' === $setting['click'] || 'true' === $setting['enable-embedded'];
				if ($has_display_option_selected) {
					$display_detail = str_replace("{{display-page-selection}}", '', $display_detail);
				} else {
					$display_detail = str_replace("{{display-page-selection}}", 'style="display:none;"', $display_detail);
				}

				$display_detail = str_replace("{{thank-you-page-selection}}", self::generate_checkbox_list($id, 'thank-you', 'page', $setting['thank-you'], $page_template, $pages), $display_detail);

				$display_detail = str_replace("{{include-page-selection}}", self::generate_page_selection_html($page_selection_template, $id, 'include', $setting['include'], $page_template, $pages, $categories, $custom_page_types, $custom_page_templates, $custom_pages), $display_detail);
				$display_detail = str_replace("{{include-post-selection}}", self::generate_post_selection_html($post_selection_template, $id, 'include', $setting['include'], $post_template, $posts, $categories, $custom_post_types, $custom_post_templates, $custom_posts), $display_detail);
				$display_detail = str_replace("{{exclude-page-selection}}", self::generate_page_selection_html($page_selection_template, $id, 'exclude', $setting['exclude'], $page_template, $pages, $categories, $custom_page_types, $custom_page_templates, $custom_pages), $display_detail);
				$display_detail = str_replace("{{exclude-post-selection}}", self::generate_post_selection_html($post_selection_template, $id, 'exclude', $setting['exclude'], $post_template, $posts, $categories, $custom_post_types, $custom_post_templates, $custom_posts), $display_detail);
				
				foreach(self::$display_setting_template_embedded_locations as $location) {
					$display_detail = str_replace("{{embedded-location-" . $location . "}}", ($setting['embedded-location'] == $location ? 'selected="selected"' : ''), $display_detail);
				}
				foreach(self::$display_setting_template_display_types as $type) {
					$display_detail = str_replace("{{" . $type . "-warning-templates}}", $display_type_warning_templates[$type], $display_detail);
					if ('true' === $style[$id]['advanced'] || false === strpos($display_type_warning_templates[$type], $style[$id]['selected-template'])) {
						$display_detail = str_replace("{{" . $type . "-initial-warning-class}}", '', $display_detail);
					} else {
						$display_detail = str_replace("{{" . $type . "-initial-warning-class}}", 'class="template-select-warning"', $display_detail);
					}
				}
				echo $display_detail;
			}
		}
		private static function generate_post_checkbox_template($posts) {
			$post_template = '';
			if ($posts) {
				foreach ($posts as $post) {
					$post_template .= '<li>' .
						'<input class="{{type}}-post-{{id}}" {{checked-' . $post->ID . '}} id="{{type}}-{{id}}-' . $post->ID . '" type="checkbox" value="true" name="[{{id}}][{{type}}][' . $post->ID . ']"><label for="{{type}}-{{id}}-' . $post->ID . '">' . esc_attr($post->post_title) . ' (' . $post->ID . ')</label>' .
					'</li>';
				}
			}
			return $post_template;
		}
		private static function generate_page_checkbox_template($pages) {
			$depth = array();
			$page_template = '';
			if ($pages) {
				foreach ($pages as $page) {
					if (0 === $page->post_parent) {
						if (count($depth) > 0) {
							while(count($depth) > 0) {
								array_pop($depth);
								$page_template .= '</ul></li>';
							}
						} else {
							$page_template .= '</li>';
						}
					} elseif (end($depth) === $page->post_parent) {
					} elseif (in_array($page->post_parent, $depth)) {
						while(end($depth) !== $page->post_parent) {
							array_pop($depth);
							$page_template .= '</ul></li>';
						}
					} else {
						$depth []= $page->post_parent;
						$page_template .= '<ul>';
					}
					$page_template .= '<li>' .
						'<input class="{{type}}-page-{{id}}" {{checked-' . $page->ID . '}} id="{{type}}-{{id}}-' . $page->ID . '" type="checkbox" value="true" name="[{{id}}][{{type}}][' . $page->ID . ']"><label for="{{type}}-{{id}}-' . $page->ID . '">' . esc_attr($page->post_title) . ' (' . $page->ID . ')</label>';
				}
			}

			if (count($depth) > 0) {
				while(count($depth) > 0) {
					array_pop($depth);
					$page_template .= '</ul></li>';
				}
			} else {
				$page_template .= '</li>';
			}
			return $page_template;
		}
		public static function generate_checkbox_list($id, $type, $post_type, $checked_status, $template, $posts) {
			$result = str_replace('{{id}}', $id, $template);
			$result = str_replace('{{type}}', $type, $result);
			$result = str_replace('{{post-type}}', $post_type, $result);
			if ($posts) {
				foreach($posts as $post) {
					if (isset($checked_status[$post->ID])) {
						$result = str_replace('{{checked-' . $post->ID . '}}', 'checked', $result);
					} else {
						$result = str_replace('{{checked-' . $post->ID . '}}', '', $result);
					}
				}
			}
			return $result;
		}
		private static function generate_signup_html_generated_field($id, $setting) {
			$generated_field = '';
			if (isset($setting['other-form-fields-name'])) {
				foreach($setting['other-form-fields-name'] as $field_id => $name) {
					$generated_field .= '<input class="sign-up-form-generated-' . $id . '" type="hidden" name="[' . $id . '][other-form-fields-name][' . $field_id . ']" value="' . esc_attr($name) . '" />';
					$generated_field .= '<input class="sign-up-form-generated-' . $id . '" type="hidden" name="[' . $id . '][other-form-fields-value][' . $field_id . ']" value="' . esc_attr($setting['other-form-fields-value'][$field_id]) . '" />';
				}
			}
			if (isset($setting['hidden-form-fields-name'])) {
				foreach($setting['hidden-form-fields-name'] as $field_id => $name) {
					$generated_field .= '<input class="sign-up-form-generated-' . $id . '" type="hidden" name="[' . $id . '][hidden-form-fields-name][' . $field_id . ']" value="' . esc_attr($name) . '" />';
					$generated_field .= '<input class="sign-up-form-generated-' . $id . '" type="hidden" name="[' . $id . '][hidden-form-fields-value][' . $field_id . ']" value="' . esc_attr($setting['hidden-form-fields-value'][$field_id]) . '" />';
				}
			}
			return $generated_field;
		}
		private static function generate_signup_html_field_selection($setting, $selected_attr) {
			if (isset($setting['other-form-fields-name'])) {
				$selection_code = '<option value=""></option>';
				foreach($setting['other-form-fields-name'] as $field_id => $name) {
					$esc_name = esc_attr($name);
					$selection_code .= '<option value="' . $esc_name . '" '. ($setting[$selected_attr] == $name ? 'selected="selected"' : '') . '>' . $esc_name . '</option>';
				}
			} else {
				$selection_code = '<option value=":null:"></option>';
			}
			return $selection_code;
		}
		public static function show_style_settings() {
			$style = self::get_style_settings();

			self::load_fonts();
			echo '<h3>Style Settings</h3>';

			$style_detail_template = file_get_contents(dirname(__FILE__) . '/resource/backend/setting-style-popup-template.php');
			foreach ($style as $id => $setting) {
				$style_detail = $style_detail_template;
				foreach(self::$style_settings_template_replace as $replace) {
					$style_detail = str_replace("{{{$replace}}}", esc_attr($setting[$replace]), $style_detail);
				}
				$style_detail = str_replace("{{id}}", $id, $style_detail);
				$style_detail = str_replace("{{selected_item_opened}}", $setting['is-open'] === 'true'?'popupally-item-opened':'', $style_detail);
				$style_detail = str_replace("{{selected_item_checked}}", $setting['is-open'] === 'true'?'checked':'', $style_detail);
				$style_detail = str_replace("{{selected_item_false_hide}}", $setting['is-open'] !== 'true'?'style="display:none;"':'', $style_detail);
				$style_detail = str_replace("{{selected_item_true_hide}}", $setting['is-open'] === 'true'?'style="display:none;"':'', $style_detail);

				$generated_fields = self::generate_signup_html_generated_field($id, $setting);
				$style_detail = str_replace("{{generated_fields}}", $generated_fields, $style_detail);
				$style_detail = str_replace("{{form-valid-false-hide}}", empty($generated_fields) ? 'style="display:none"' : '', $style_detail);

				$style_detail = str_replace("{{signup_name_field_selection}}", self::generate_signup_html_field_selection($setting, 'sign-up-form-name-field'), $style_detail);
				$style_detail = str_replace("{{signup_lname_field_selection}}", self::generate_signup_html_field_selection($setting, 'sign-up-form-lname-field'), $style_detail);
				$style_detail = str_replace("{{signup_email_field_selection}}", self::generate_signup_html_field_selection($setting, 'sign-up-form-email-field'), $style_detail);

				// initial visibilility generation
				$style_detail = str_replace("{{advanced-true-hide}}", $setting['advanced'] === 'true'?'style="display:none;"':'', $style_detail);
				$style_detail = str_replace("{{advanced-false-hide}}", $setting['advanced'] !== 'true'?'style="display:none;"':'', $style_detail);
				$style_detail = str_replace("{{signup-raw-html-form-hide}}", strpos('bxsjbi,plsbvs,cjthhv,tozpom,vtgjid,rwnkmg', $setting['selected-template'])?'':'style="display:none;"', $style_detail);

				$template_selection_code = '';
				$template_customization_code = '';
				foreach (PopupAllyPro::$available_templates as $template_uid => $template_obj) {
					$selected = $template_uid === $setting['selected-template'];
					$template_selection_code .= '<option value="' . $template_uid . '" ' . ($selected ? 'selected="selected"' : '') . '>' . esc_attr($template_obj->template_name) . '</option>';
					$template_customization_code .= '<div ' . ($selected ? '' : 'style="display:none;"') . ' hide-toggle data-dependency="template-selector-' . $id . '" data-dependency-value="' . $template_uid . '" >';
					$template_customization_code .= $template_obj->show_style_settings($id, $setting);
					$template_customization_code .= '</div>';
				}
				$style_detail = str_replace("{{template_selection}}", $template_selection_code, $style_detail);
				$style_detail = str_replace("{{template_customization}}", $template_customization_code, $style_detail);

				foreach(self::$style_setting_template_signup_html as $type) {
					$template_obj = self::$available_templates[$style[$id]['selected-template']];
					if (false === $template_obj->style_hide_signup_fields[$type]) {
						$style_detail = str_replace("{{signup-html-template-" . $type . "-initial-hide}}", '', $style_detail);
					} elseif (true === $template_obj->style_hide_signup_fields[$type]) {
						$style_detail = str_replace("{{signup-html-template-" . $type . "-initial-hide}}", 'style="display:none;"', $style_detail);
					} elseif ('true' === $setting[$template_obj->style_hide_signup_fields[$type]]) {
						$style_detail = str_replace("{{signup-html-template-" . $type . "-initial-hide}}", 'style="display:none;"', $style_detail);
					} else {
						$style_detail = str_replace("{{signup-html-template-" . $type . "-initial-hide}}", '', $style_detail);
					}
				}
				echo $style_detail;
			}
		}

		public static function show_copy_delete_settings() {
			$style = self::get_style_settings();
			include (dirname(__FILE__) . '/resource/backend/setting-copy-delete.php');
		}

		public static function show_advanced_settings() {
			$plugin_dir = plugin_dir_url(__FILE__);
			$advanced = self::get_advanced_settings();
			include (dirname(__FILE__) . '/resource/backend/setting-advanced.php');
		}

		public static function show_import_export_settings() {
			$style = self::get_style_settings();
			$nonce_download_url = add_query_arg(array('export-popupally-nonce' => wp_create_nonce("popupally-export")), admin_url('admin.php'));
			include (dirname(__FILE__) . '/resource/backend/setting-import-export.php');
		}

		public static function show_license_settings() {
			$license = self::get_license_settings();
			include (dirname(__FILE__) . '/resource/backend/setting-license.php');
		}

		public static function sanitize_settings($input) {
			if (!isset($input[self::SETTING_KEY_LICENSE])) {
				return $input;
			}
			if (self::$popupally_pro_enabled) {
				$display = self::convert_setting_string_to_array($input[self::SETTING_KEY_DISPLAY]);
				$style = self::convert_setting_string_to_array($input[self::SETTING_KEY_STYLE]);
				$copy_delete = self::convert_setting_string_to_array($input[self::SETTING_KEY_COPY_DELETE]);
				$advanced = self::convert_setting_string_to_array($input[self::SETTING_KEY_ADVANCED]);
				$add = self::convert_setting_string_to_array($input['add']);
				if (false === $display || false === $style || false === $copy_delete || false === $advanced || false === $add) {
					add_settings_error('popupally_pro_settings', 'settings_updated', 'Setting update failed!', 'error');
					return $input['selected'];
				}
			}
			$license = self::convert_setting_string_to_array($input[self::SETTING_KEY_LICENSE]);
			if (false === $license) {
				add_settings_error('popupally_pro_settings', 'settings_updated', 'Setting update failed!', 'error');
				return $input['selected'];
			}

			if (self::$popupally_pro_enabled) {
				self::sanitize_display_settings($display);
				self::sanitize_style_settings($style);
				self::sanitize_copy_delete_settings($copy_delete);
				if (isset($add['add-new']) && $add['add-new'] === 'true' && isset($add['num-new'])) {
					$num = intval($add['num-new']);
					if ($num > 0) {
						self::add_new_popups($num);
					}
				}
				self::get_popup_code(true);
				self::sanitize_advanced_settings($advanced);

				set_transient(self::SETTING_KEY_ALL, $input['selected'], self::CACHE_PERIOD);
			}
			self::sanitize_license_settings($license, $input['selected']['selected-tab'] === 'license');
			add_settings_error('popupally_pro_settings', 'settings_updated', 'Settings saved!', 'updated');
			return $input['selected'];
		}
		private static function sanitize_display_settings($input) {
			$to_remove = array();	// used to remove any unwanted <input> that got captured by the serialization
			foreach ($input as $id => &$setting) {
				if (is_int($id)) {
					$setting = wp_parse_args($setting, self::get_default_popup_display_setting($id));
					$setting['timed-popup-delay'] = intval($setting['timed-popup-delay']);
					$setting['scroll-percent'] = intval($setting['scroll-percent']);
					$setting['priority'] = intval($setting['priority']);
					$setting['cookie-duration'] = intval($setting['cookie-duration']);

					$parts = explode(',', $setting['utm-source']);
					$parts = array_map('trim', $parts);
					$setting['utm-source'] = implode(',', $parts);
				} else {
					$to_remove []= $id;
				}
			}
			foreach ($to_remove as $id) {
				unset($input[$id]);
			}
			update_option(self::SETTING_KEY_DISPLAY, $input);
			set_transient(self::SETTING_KEY_DISPLAY, $input, self::CACHE_PERIOD);
			return $input;
		}
		private static function sanitize_style_settings($input) {
			$to_remove = array();	// used to remove any unwanted <input> that got captured by the serialization
			foreach ($input as $id => &$setting) {
				if (is_int($id)) {
					if ('true' === $setting['advanced']) {
						continue;
					}
					foreach(self::$available_templates as $key => $template_ob) {
						$setting = $template_ob->sanitize_style($setting, $id, $setting['selected-template'] === $key);
					}
					$setting = wp_parse_args($setting, self::get_default_popup_style_setting($id));
					if ($setting['sign-up-form-name-field'] === ':null:') {
						$setting['sign-up-form-name-field'] = '';
					}
					if ($setting['sign-up-form-lname-field'] === ':null:') {
						$setting['sign-up-form-lname-field'] = '';
					}
					if ($setting['sign-up-form-email-field'] === ':null:') {
						$setting['sign-up-form-email-field'] = '';
					}
					if (isset($setting['convert-advanced']) && 'true' === $setting['convert-advanced']) {
						$setting['html'] = self::generate_popup_html($id, $setting, $input);
						$setting['html-embedded'] = self::generate_popup_html($id, $setting, $input, 1);
						$setting['css'] = self::generate_popup_css($id, $setting, $input);
						$setting['css-top-margin'] = self::generate_popup_css($id, $setting, $input, 2);
						$setting['advanced'] = 'true';
					}
				} else {
					$to_remove []= $id;
				}
			}
			foreach ($to_remove as $id) {
				unset($input[$id]);
			}
			self::increment_num_style_saved($input);
			update_option(self::SETTING_KEY_STYLE, $input);
			set_transient(self::SETTING_KEY_STYLE, $input, self::CACHE_PERIOD);
		}
		private static function sanitize_copy_delete_settings($input) {
			$to_remove = array();
			$to_copy = array();
			foreach ($input as $id => &$setting) {
				if (is_int($id)) {
					if (isset($setting['delete'])) {
						$to_remove []= $id;
						continue;
					}
					if (isset($setting['copy'])) {
						unset($setting['copy']);
						$to_copy[$id] = $setting['copy-name'];
					}
				}
			}
			if (!empty($to_remove)) {
				self::remove_popups($to_remove);
			}
			foreach($to_copy as $id => $new_name) {
				self::copy_popup($id, $new_name);
			}
		}

		private static function copy_popup($source_id, $new_name) {
			$display = self::get_display_settings();
			$style = self::get_style_settings();

			$new_id = max(array_keys($display)) + 1;
			$display[$new_id] = $display[$source_id];
			$style[$new_id] = $style[$source_id];
			$style[$new_id]['name'] = empty($new_name) ? 'Popup ' . $new_id : $new_name;

			update_option(self::SETTING_KEY_DISPLAY, $display);
			update_option(self::SETTING_KEY_STYLE, $style);
			set_transient(self::SETTING_KEY_DISPLAY, $display, self::CACHE_PERIOD);
			set_transient(self::SETTING_KEY_STYLE, $style, self::CACHE_PERIOD);
		}

		private static function add_new_popups($num) {
			$display = self::get_display_settings();
			$style = self::get_style_settings();

			if (empty($display)) {
				$start = 1;
			} else {
				$start = max(array_keys($display)) + 1;
			}
			for ($i = 0; $i < $num; ++$i) {
				$display[$start + $i] = self::get_default_popup_display_setting($start + $i);
				$style[$start + $i] = self::get_default_popup_style_setting($start + $i);
			}
			update_option(self::SETTING_KEY_DISPLAY, $display);
			update_option(self::SETTING_KEY_STYLE, $style);
			set_transient(self::SETTING_KEY_DISPLAY, $display, self::CACHE_PERIOD);
			set_transient(self::SETTING_KEY_STYLE, $style, self::CACHE_PERIOD);
		}

		private static function remove_popups($to_remove) {
			$display = self::get_display_settings();
			$style = self::get_style_settings();
			foreach ($to_remove as $id) {
				unset($display[$id]);
				unset($style[$id]);
			}
			update_option(self::SETTING_KEY_STYLE, $style);
			update_option(self::SETTING_KEY_DISPLAY, $display);
			set_transient(self::SETTING_KEY_DISPLAY, $display, self::CACHE_PERIOD);
			set_transient(self::SETTING_KEY_STYLE, $style, self::CACHE_PERIOD);
		}
		private static function sanitize_advanced_settings($input) {
			$input = wp_parse_args($input, self::get_default_advanced_setting());
			if ('true' === $input['no-inline']) {
				self::generate_script_files();
			}
			update_option(self::SETTING_KEY_ADVANCED, $input);
			set_transient(self::SETTING_KEY_ADVANCED, $input, self::CACHE_PERIOD);
		}
		private static function sanitize_license_settings($input, $force = false) {
			$input['email'] = trim($input['email']);
			$input['serial'] = trim($input['serial']);
			if ($force || $input['old-email'] !== $input['email'] || $input['old-serial'] != $input['serial']) {
				PopupAllyProUpdater::get_plugin_update(true, $input);
			}
			unset($input['old-email']);
			unset($input['old-serial']);
			update_option(self::SETTING_KEY_LICENSE, $input);
			set_transient(self::SETTING_KEY_LICENSE, $input, self::CACHE_PERIOD);
		}

		private static function increment_num_style_saved($style) {			
			$num_saved = self::get_num_style_saved_settings();
			$num_saved += 1;
			update_option(self::SETTING_KEY_NUM_STYLE_SAVED, $num_saved);
			set_transient(self::SETTING_KEY_NUM_STYLE_SAVED, $num_saved, self::CACHE_PERIOD);
		}

		private static function generate_popup_code() {
			$display = self::get_display_settings();
			$style = self::get_style_settings();

			$code = array('version' => self::VERSION);
			foreach($display as $id => $display_setting) {
				$code[$id] = array();
				$code[$id]['html'] = self::generate_popup_html($id, $style[$id], $style, 0);
				$code[$id]['embedded_html'] = self::generate_popup_html($id, $style[$id], $style, 1);
				$code[$id]['css'] = self::generate_popup_css($id, $style[$id], $style, 0);
				$code[$id]['top_margin_css'] = self::generate_popup_css($id, $style[$id], $style, 2);

				$code[$id]['preview_html'] = array();
				$code[$id]['preview_css'] = array();
				foreach(self::$available_templates as $template_uid => $template_obj) {
					$code[$id]['preview_html'][$template_uid] = self::generate_popup_html($id, $style[$id], $style, 2, $template_obj);
					$code[$id]['preview_css'][$template_uid] = self::generate_popup_css($id, $style[$id], $style, 1, $template_obj);
				}
				$code[$id]['thank_you'] = self::generate_thank_you_js($style[$id]);
			}
			return $code;
		}

		private static function generate_script_files() {
			if (false === ($creds = request_filesystem_credentials('admin.php', '', false, false, null))) {
				return true;
			}
			if (!WP_Filesystem($creds)) {
				echo 'Cannot initiate WP_Filesystem. Please make sure you have the proper permission on the WordPress Install';
				return true;
			}
			global $wp_filesystem;
			$target_dir = trailingslashit($wp_filesystem->wp_content_dir());
			$target_dir = trailingslashit($target_dir . self::SCRIPT_FOLDER);

			$wp_filesystem->mkdir($target_dir);

			$code = self::get_popup_code();
			foreach($code as $id => $values){
				if (is_array($values)) {	// do not process the 'version' key
					$css .= $values['css'];

					$thank_you = 'var exdate = new Date();exdate.setFullYear(exdate.getFullYear() + 1);' . $values['thank_you'];
					$wp_filesystem->put_contents($target_dir . 'popupally-pro-thank-you-' . $id . '.js', $thank_you, FS_CHMOD_FILE);
				}
			}
			$wp_filesystem->put_contents($target_dir . 'popupally-pro-style.css', $css, FS_CHMOD_FILE);
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Front end">
		public static function add_popup_scripts() {
			$advanced = self::get_advanced_settings();
			if ('true' === $advanced['no-inline']) {
				return;
			}
			$to_show = self::get_popup_to_show();
			$dependencies = self::get_popup_dependencies($to_show);
			$code = self::get_popup_code();
			if (!empty($to_show)) {
				$display = self::get_display_settings();
				echo '<script type="text/javascript">';
				readfile(dirname(__FILE__) . '/resource/frontend/popup.min.js');
				echo '</script>';
				echo '<style type="text/css">';
				foreach($to_show as $id => $value){
					echo $code[$id]['css'];
					if (in_array('embedded', $value)) {
						if ('top-page' === $display[$id]['embedded-location'] || 'top-page-follow' === $display[$id]['embedded-location']) {
							echo $code[$id]['top_margin_css'];
						}
					}
				}
				foreach($dependencies as $id) {
					echo $code[$id]['css'];
				}
				echo '</style>';
			}
			$ids = self::get_popup_thank_you_to_show();
			if (!empty($ids)){
				echo '<script type="text/javascript">var exdate = new Date();exdate.setFullYear(exdate.getFullYear() + 1);';
				foreach($ids as $id) {
					echo $code[$id]['thank_you'];
				}
				echo '</script>';
			}
		}
		public static function add_preview_popup_scripts() {
			$code = self::get_popup_code();
			echo '<style type="text/css">';
			foreach($code as $id => $values) {
				if (is_array($values)) {
					foreach($values['preview_css'] as $css) {
						echo $css;
					}
				}
			}
			echo '</style>';
		}
		private static function generate_popup_parameters($display, $style, $popup_types = false) {
			$param = array();
			if (false !== $popup_types) {
				foreach($popup_types as $type) {
					switch($type) {
						case 'timed':
							$param['timed-popup-delay'] = $display['timed-popup-delay'];
							break;
						case 'exit-intent':
							$param['enable-exit-intent-popup'] = $display['enable-exit-intent-popup'];
							break;
						case 'scroll':
							$param['enable-scroll-popup'] = $display['scroll'];
							$param['scroll-percent'] = $display['scroll-percent'];
							$param['scroll-trigger'] = esc_attr($display['scroll-trigger']);
							break;
						case 'click':
							$param['open-trigger'] = esc_attr($display['open-trigger']);
							break;
					}
				}
			}
			self::construct_html_parameter_string($display, self::$config_display_settings, $param);
			self::construct_html_parameter_string($style, self::$config_style_settings, $param);
			return $param;
		}
		public static function add_popup_html() {
			$to_show = self::get_popup_to_show();
			if (!empty($to_show)) {
				$display = self::get_display_settings();
				$style = self::get_style_settings();
				$code = self::get_popup_code();
				foreach($to_show as $id => $popup_types) {
					$param = self::generate_popup_parameters($display[$id], $style[$id], $popup_types);
					echo '<div class="popupally-configuration" style="display:none;">' . json_encode(self::replace_json_safe_string($param)) . '</div>';
					echo $code[$id]['html'];
					if (in_array('embedded', $popup_types)) {
						if ('page-end' === $display[$id]['embedded-location']) {
							echo $code[$id]['embedded_html'];
						} elseif ('top-page' === $display[$id]['embedded-location']) {
							$temp = str_replace('class-placeholder-jehjsq', 'popupally-pro-top-page-jehjsq', $code[$id]['embedded_html']);
							$temp = str_replace('role-placeholder-kjdshe', 'role="navigation"', $temp);
							echo $temp;
						} elseif ('top-page-follow' === $display[$id]['embedded-location']) {
							$temp = str_replace('class-placeholder-jehjsq', 'popupally-pro-top-page-follow-jehjsq', $code[$id]['embedded_html']);
							$temp = str_replace('role-placeholder-kjdshe', 'role="navigation"', $temp);
							echo $temp;
						} elseif ('page-end-follow' === $display[$id]['embedded-location']) {
							$temp = str_replace('class-placeholder-jehjsq', 'popupally-pro-end-page-follow-jehjsq', $code[$id]['embedded_html']);
							echo $temp;
						}
					}
				}

				$dependencies = self::get_popup_dependencies($to_show);
				foreach($dependencies as $id) {
					$param = self::generate_popup_parameters($display[$id], $style[$id]);
					echo '<div class="popupally-configuration" style="display:none;">' . json_encode(self::replace_json_safe_string($param)) . '</div>';
					echo $code[$id]['html'];
				}
			}
		}

		// 0 - normal popup, 1 - embedded, 2 - preview
		private static function generate_popup_html($id, $setting, $all_style, $mode = 0, $template_obj = null) {
			if (null === $template_obj && 'true' === $setting['advanced']) {
				if (2 === $mode){
					return '';
				} elseif (1 === $mode){
					return $setting['html-embedded'];
				} else {
					return $setting['html'];
				}
			}
			if (2 !== $mode) {
				if (isset(self::$available_templates[$setting['selected-template']])) {
					$template_obj = &self::$available_templates[$setting['selected-template']];
				} else {
					$template_obj = &self::$available_templates['bxsjbi'];
				}
			}

			return $template_obj->generate_popup_html($id, $setting, $all_style, $mode);
		}

		// 0 - normal popup, 1 - embedded, 2 - preview
		private static function generate_popup_css($id, $setting, $all_style, $mode = 0, $template_obj = null) {
			if ('true' === $setting['advanced']) {
				if ($mode === 2) {
					return $setting['css-top-margin'];
				}
				return $setting['css'];
			}

			if (null === $template_obj) {
				if (isset(self::$available_templates[$setting['selected-template']])) {
					$template_obj = &self::$available_templates[$setting['selected-template']];
				} else {
					$template_obj = &self::$available_templates['bxsjbi'];
				}
			}
			return $template_obj->generate_popup_css($id, $setting, $all_style, $mode);
		}

		private static function generate_thank_you_js($style) {
			return 'document.cookie = "' . $style['cookie-name'] . '=1; path=/; expires="+ exdate.toGMTString();';
		}

		public static function get_popup_thank_you_to_show($post_id = false) {
			if ($post_id === false) {
				global $wp_query;
				if (isset($wp_query) && isset($wp_query->post)) {
					$post_id = $wp_query->post->ID;
				} else {
					return array();
				}
			}
			$cookies = array();
			$display = self::get_display_settings();
			foreach ($display as $id => $settings) {
				if ((isset($settings['timed']) && 'true' === $settings['timed']) ||
						(isset($settings['enable-exit-intent-popup']) && 'true' === $settings['enable-exit-intent-popup']) ||
						(isset($settings['scroll']) && 'true' === $settings['scroll']) ||
						(isset($settings['click']) && 'true' === $settings['click'])) {
					if (isset($settings['thank-you'][$post_id])) {
						$cookies []= $id;
					}
				}
			}
			return $cookies;
		}

		private static function get_post_categories($post_id) {
			$post_categories = wp_get_post_categories($post_id);
			$categories = array();
			foreach($post_categories as $c){
				$cat = get_category($c);
				$categories[$cat->cat_ID] = $cat->slug;
			}
			return $categories;
		}
		private static function check_post_selection($post_id, $is_front_page, $category_id, $post_type, $categories, $selection) {
			if ('page' === $post_type && isset($selection['all-pages'])) {
				return true;
			}
			if ('post' === $post_type && isset($selection['all-posts'])) {
				return true;
			}
			if (false !== $post_id) {
				if (isset($selection['all-'.$post_type])) {
					return true;
				}
				if (isset($selection[$post_id])) {
					return true;
				}
				foreach($categories as $id => $slug) {
					if (isset($selection['category-post-' . $id])) {
						return true;
					}
				}
			} elseif ($is_front_page && isset($selection['front-page'])) {
				return true;
			} elseif ($category_id >= 0 && isset($selection['category-' . $category_id])) {
				return true;
			}
			return false;
		}
		private static function get_popup_dependencies($to_show) {
			$style = self::get_style_settings();
			$already_included = array_keys($to_show);
			$dependencies = array();
			foreach($to_show as $id => $popup_types) {
				self::get_popup_dependency($style, $already_included, $dependencies, $id);
			}
			return $dependencies;
		}
		private static function get_popup_dependency($style, &$already_included, &$dependencies, $to_include) {
			if (isset(self::$available_templates[$style[$to_include]['selected-template']])) {
				$template_obj = &self::$available_templates[$style[$to_include]['selected-template']];
				if ($template_obj) {
					$dependency = $template_obj->get_popup_dependency($style[$to_include]);
					if (false !== $dependency) {
						foreach($dependency as $id) {
							if (!in_array($id, $already_included)) {
								$already_included []= $id;
								$dependencies []= $id;
								self::get_popup_dependency($style, $already_included, $dependencies, $id);
							}
						}
					}
				}
			}
		}
		public static function get_popup_to_show($post_id = false, $ignore_disable_mobile = false) {
			$is_front_page = false;
			$category_id = -1;
			if ($post_id === false) {
				global $wp_query;
				if (!isset($wp_query)) {
					return array();
				}
				if ($wp_query->is_home) {
					$is_front_page = true;
					$post_type = 'page';
				} elseif ($wp_query->is_category) {
					$category_id = $wp_query->queried_object_id;
					$post_type = 'page';
				} elseif (isset($wp_query->post)) {
					$post_id = $wp_query->post->ID;
					$post_type = $wp_query->post->post_type;
					$categories = self::get_post_categories($post_id);
				} else {
					return array();
				}
			} else {
				$post = get_post($post_id);
				if (null === $post) {
					return array();
				}
				$post_type = $post->post_type;
				$categories = self::get_post_categories($post_id);
			}
			$timed = 0;
			$exit_intent = 0;
			$scroll = 0;
			$result = array();
			$display = self::get_display_settings();
			$is_mobile = $ignore_disable_mobile ? false : wp_is_mobile();
			foreach ($display as $id => $settings) {
				if (false !== $post_id && isset($settings['thank-you'][$post_id])) {
					continue;
				}
				if ($is_mobile && 'true' ===$settings['disable-mobile']) {
					continue;
				}
				$to_show = false;
				if (isset($settings['show-all']) && 'true' === $settings['show-all']) {	// exclude path
					$to_show = !self::check_post_selection($post_id, $is_front_page, $category_id, $post_type, $categories, $settings['exclude']);
				} else {	// include path
					$to_show = self::check_post_selection($post_id, $is_front_page, $category_id, $post_type, $categories, $settings['include']);
				}
				if (!$to_show) {
					continue;
				}
				$row = array();
				if ('true' === $settings['click']) {
					$row []= 'click';
				}
				if ($timed <= 0 && 'true' === $settings['timed']) {
					$timed = $id;
					$row []= 'timed';
				}
				if ($exit_intent <= 0 && 'true' === $settings['enable-exit-intent-popup']) {
					$exit_intent = $id;
					$row []= 'exit-intent';
				}
				if ($scroll <= 0 && 'true' === $settings['scroll']) {
					$scroll = $id;
					$row []= 'scroll';
				}
				if ('true' === $settings['enable-embedded']) {
					$row []= 'embedded';
				}
				if (!empty($row)) {
					$result[$id] = $row;
				}
			}
			return $result;
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Export and import">
		public static function process_export() {
			if (isset($_REQUEST['id']) && isset($_REQUEST['export-popupally-nonce']) && wp_verify_nonce($_REQUEST['export-popupally-nonce'], "popupally-export")) {
				$id = intval($_REQUEST['id']);
				set_time_limit(0);
				$display = self::get_display_settings();
				$style = self::get_style_settings();
				$filename = "$id - " . $style[$id]['name'] . ".popupally";

				header('Content-Type: text/csv');
				header(sprintf('Content-Disposition: attachment; filename="%s"', $filename));

				$result = array('display' => $display[$id], 'style' => $style[$id]);
				echo json_encode(self::replace_json_safe_string($result));
				exit;
			}
		}
		// </editor-fold>
		
		// <editor-fold defaultstate="collapsed" desc="License checking">
		private static function check_license_status() {
			$enabled = self::get_enable_setting();
			if (!is_array($enabled)) {
				PopupAllyProUpdater::get_plugin_update(true);
				$enabled = self::get_enable_setting();
			}
			if (!is_array($enabled)) {
				self::$popupally_pro_enabled = false;
			} else {
				self::$popupally_pro_enabled = $enabled['enabled'];
			}
		}

		private static function get_enable_setting() {
			$enabled = get_transient(self::SETTING_KEY_ENABLED);

			if (!is_array($enabled)) {
				$enabled = get_option(self::SETTING_KEY_ENABLED, false);

				set_transient(self::SETTING_KEY_ENABLED, $enabled, self::CACHE_PERIOD);
			}
			return $enabled;
		}

		public static function set_enable_setting($is_enabled = false) {
			$enabled = array('enabled' => $is_enabled);
			update_option(self::SETTING_KEY_ENABLED, $enabled);
			set_transient(self::SETTING_KEY_ENABLED, $enabled, self::CACHE_PERIOD);
		}
		// </editor-fold>

		// <editor-fold defaultstate="collapsed" desc="Utlities">
		private static function json_name_string_to_keys($name) {
			$a = array();
			$start = strpos($name, '[');
			$end = 0;
			while($start !== false) {
				$start += 1;
				$end = strpos($name, ']', $start);
				if ($end === false) {
					$end = $start;
					break;
				}
				$a []= substr($name, $start, $end - $start);
				$start = strpos($name, '[', $end);
			}
			if ($end < strlen($name)-1) {
				$a []= substr($name, $end);
			}
			return $a;
		}
		private static function convert_setting_string_to_array($str) {
			$raw_array = json_decode($str, true);
			$a = array();
			if (!is_array($raw_array)) {
				return false;
			}
			foreach ($raw_array as $tuple) {
				$keys = self::json_name_string_to_keys($tuple['name']);
				$v = & $a;
				for($i = 0; $i < count($keys) - 1; ++$i) {
					if (!array_key_exists($keys[$i], $v)) {
						$v[$keys[$i]] = array();
					}
					$v = & $v[$keys[$i]];
				}
				$v[end($keys)] = $tuple['value'];
			}
			return $a;
		}
		private static function replace_json_safe_string($param) {
			foreach ($param as $key => $value) {
				if (is_string($value)) {
					$value = str_replace('&quot;', '"', $value);
					$value = str_replace('&#039;', "'", $value);
					$value = str_replace('&lt;', '<', $value);
					$value = str_replace('&gt;', '>', $value);
					$value = str_replace('&amp;', '&', $value);
					$param[$key] = $value;
				}
			}
			return $param;
		}

		public static function get_display_settings() {
			$display = get_transient(self::SETTING_KEY_DISPLAY);

			if (!is_array($display)) {
				$display = get_option(self::SETTING_KEY_DISPLAY, self::get_default_display_setting());

				set_transient(self::SETTING_KEY_DISPLAY, $display, self::CACHE_PERIOD);
			}
			if (!is_array($display)) {
				$display = self::get_default_display_setting();
			}
			foreach($display as $id => $setting) {
				if (is_int($id)) {
					$display[$id] = wp_parse_args($display[$id], self::get_default_popup_display_setting($id));

					// ensure backwards compatibility by converting all arrays to new ones
					$display[$id]['include'] = self::convert_array_list($display[$id]['include']);
					$display[$id]['exclude'] = self::convert_array_list($display[$id]['exclude']);
					$display[$id]['thank-you'] = self::convert_array_list($display[$id]['thank-you']);
				}
			}

			return $display;
		}

		private static function convert_array_list($list) {
			if (!is_array($list)) {
				return array();
			}
			if (empty($list) || reset($list) === 'true') {
				return $list;
			}
			$result = array();
			foreach($list as $id) {
				$result[$id] = 'true';
			}
			return $result;
		}

		public static function get_style_settings() {
			$style = get_transient(self::SETTING_KEY_STYLE);

			if (!is_array($style)) {
				$style = get_option(self::SETTING_KEY_STYLE, self::get_default_style_setting());

				set_transient(self::SETTING_KEY_STYLE, $style, self::CACHE_PERIOD);
			}
			if (!is_array($style)) {
				$style = self::get_default_style_setting();
			}
			// update old setting to new ones
			foreach($style as $id => $setting) {
				if (is_int($id)) {
					if ('true' !== $style[$id]['advanced'] && isset($style[$id]['popup-selector'])) {
						unset($style[$id]['popup-selector']);
						unset($style[$id]['popup-class']);
						unset($style[$id]['close-trigger']);
						unset($style[$id]['cookie-name']);
					}
					if (!isset($style[$id]['sign-up-form-valid'])) {
						if (isset($style[$id]['hidden-form-fields-name']) || isset($style[$id]['other-form-fields-name'])) {
							$style[$id]['sign-up-form-valid'] = 'true';
						} else {
							$style[$id]['sign-up-form-valid'] = 'false';
						}
					}
					$style[$id] = wp_parse_args($style[$id], self::get_default_popup_style_setting($id));
				}
			}

			return $style;
		}

		private static function get_advanced_settings() {
			$advanced = get_transient(self::SETTING_KEY_ADVANCED);

			if (!is_array($advanced)) {
				$advanced = get_option(self::SETTING_KEY_ADVANCED, self::get_default_advanced_setting());

				set_transient(self::SETTING_KEY_ADVANCED, $advanced, self::CACHE_PERIOD);
			}
			$advanced = wp_parse_args($advanced, self::get_default_advanced_setting());
			return $advanced;
		}

		public static function get_license_settings() {
			$license = get_transient(self::SETTING_KEY_LICENSE);

			if (!is_array($license)) {
				$license = get_option(self::SETTING_KEY_LICENSE, self::get_default_license_setting());

				set_transient(self::SETTING_KEY_LICENSE, $license, self::CACHE_PERIOD);
			}
			$license = wp_parse_args($license, self::get_default_license_setting());
			$license['email'] = trim($license['email']);
			$license['serial'] = trim($license['serial']);
			return $license;
		}

		public static function get_selected_settings() {
			$selected = get_transient(self::SETTING_KEY_ALL);

			if (!is_array($selected)) {
				$selected = get_option(self::SETTING_KEY_ALL, array('selected-tab' => 'display'));

				set_transient(self::SETTING_KEY_ALL, $selected, self::CACHE_PERIOD);
			}
			if (!is_array($selected) || empty($selected['selected-tab'])) {
				$selected = array('selected-tab' => 'display');
			}
			return $selected;
		}

		public static function get_num_style_saved_settings() {
			$num = get_transient(self::SETTING_KEY_NUM_STYLE_SAVED);

			if (false === $num) {
				$num = get_option(self::SETTING_KEY_NUM_STYLE_SAVED, 0);

				set_transient(self::SETTING_KEY_NUM_STYLE_SAVED, $num, self::CACHE_PERIOD);
			}
			return $num;
		}

		public static function get_popup_code($force = false) {
			if ($force) {
				$code = self::generate_popup_code();
				update_option(self::SETTING_KEY_CODE, $code);
				set_transient(self::SETTING_KEY_CODE, $code, self::CACHE_PERIOD);
				return $code;
			}
			$code = get_transient(self::SETTING_KEY_CODE);

			if (!is_array($code) || !isset($code['version']) || $code['version'] !== self::VERSION) {
				$code = get_option(self::SETTING_KEY_CODE, 0);
				if (!is_array($code) || !isset($code['version']) || $code['version'] !== self::VERSION) {
					$code = self::generate_popup_code();
					update_option(self::SETTING_KEY_CODE, $code);
				}
				set_transient(self::SETTING_KEY_CODE, $code, self::CACHE_PERIOD);
			}
			return $code;
		}

		private static function get_default_display_setting() {
			if (self::$default_display_settings === null) {
				self::$default_display_settings = array(1 => self::get_default_popup_display_setting(1));
			}
			return self::$default_display_settings;
		}

		private static function get_default_style_setting() {
			if (self::$default_style_settings === null) {
				self::$default_style_settings = array(1 => self::get_default_popup_style_setting(1));
			}
			return self::$default_style_settings;
		}

		private static function get_default_advanced_setting() {
			if (self::$default_advanced_settings === null) {
				self::$default_advanced_settings = array('no-inline' => 'false', 'max-page' => '500', 'max-post' => '500');
			}
			return self::$default_advanced_settings;
		}

		private static function get_default_license_setting() {
			if (self::$default_license_settings === null) {
				self::$default_license_settings = array('email' => '', 'serial' => '');
			}
			return self::$default_license_settings;
		}

		private static function get_default_popup_display_setting($id = false) {
			if (self::$default_popup_display_settings === null) {
				self::$default_popup_display_settings = array('timed' => 'false',
					'timed-popup-delay' => -1,
					'enable-exit-intent-popup' => 'false',
					'scroll' => 'false',
					'scroll-percent' => -1,
					'scroll-trigger' => '',
					'click' => 'false',
					'fade-in' => '0',
					'enable-embedded' => 'false',
					'embedded-location' => 'none',
					'open-trigger' => '.popup-click-open-trigger-{{num}}',
					'priority' => 0,
					'show-all' => 'false',
					'include' => array(),
					'exclude' => array(),
					'cookie-duration' => 14,
					'thank-you' => array(),
					'disable-mobile' => 'false',
					'utm-source' => '',
					'is-open' => 'false',
				);
			}
			if (false === $id) {
				return self::$default_popup_display_settings;
			}
			return self::customize_parameter_array(self::$default_popup_display_settings, $id);
		}

		private static function get_default_popup_style_setting($id = false) {
			if (self::$default_popup_style_simple_settings === null) {
				self::$default_popup_style_simple_settings = array('name' => 'Popup {{num}}',
					'advanced' => 'false',
					'signup-form' => '',
					'sign-up-form-action' => '',
					'sign-up-form-valid' => 'false',
					'sign-up-form-name-field' => '',
					'sign-up-form-lname-field' => '',
					'sign-up-form-email-field' => '',
					'html' => '',
					'html-embedded' => '',
					'css' => '',
					'css-top-margin' => '',
					'selected-template' => 'bxsjbi',
					'close-iframe-selector' => '',
					'popup-selector' => '#popup-box-pro-gfcr-{{num}}',
					'popup-class' => 'popupally-pro-opened-pro-gfcr-{{num}}',
					'cookie-name' => 'popupally-cookie-{{num}}',
					'close-trigger' => '.popup-click-close-trigger-{{num}}',
					'is-open' => 'false',
					);
				foreach (self::$available_templates as $template) {
					self::$default_popup_style_simple_settings = array_merge(self::$default_popup_style_simple_settings, $template->default_values);
				}
			}
			if (false === $id) {
				return self::$default_popup_style_simple_settings;
			}
			return self::customize_parameter_array(self::$default_popup_style_simple_settings, $id);
		}

		private static function construct_html_parameter_string($source, $tags, &$target) {
			foreach ($tags as $tag) {
				if (isset($source[$tag])) {
					$target[$tag] = $source[$tag];
				} else {
					$target[$tag] = '';
				}
			}
		}

		private static function customize_parameter_array($source, $num) {
			$result = array();
			foreach ($source as $tag => $value) {
				$result[$tag] = str_replace('{{num}}', $num, $value);
			}
			return $result;
		}

		private static function check_php_version($setting = false) {
			if (!defined('PHP_VERSION_ID')) {
				$version = explode('.', PHP_VERSION);
				define('PHP_VERSION_ID', ($version[0] * 10000 + $version[1] * 100 + $version[2]));
			}
			if (PHP_VERSION_ID < 50300) {
				$message = 'The server is currently running PHP Version ' . PHP_VERSION . '. PopupAlly Pro needs at least PHP 5.3 to function properly. Please ask your host to upgrade.';
				if (false !== $setting) {
					add_settings_error($setting, 'php_version_error', $message, 'error');
				} else {
					return $message;
				}
			}
			return false;
		}
		private static function generate_dropdown_selection_template($list, $css_property) {
			$template = '<select name="{{base_variable_name}}[{{variable_name}}]" id="{{variable_name}}-font-selection-{{id}}" preview-update-target-css="{{target_name}}-{{id}}" preview-update-target-css-property="' . $css_property . '">';

			foreach($list as $value => $display) {
				$template .= '<option value="' . esc_attr($value). '" {{select-' . $value . '}}>' . esc_html($display) . '</option>';
			}
			if ('font-family' === $css_property){
				$template .= '<option value="other" {{select-other}}>Other</option>';
				$template .= '</select>';
				$template .= '<input name="{{base_variable_name}}[{{variable_name}}-other]" class="full-width" value="{{font-value}}" {{hide-other}} hide-toggle data-dependency="{{variable_name}}-font-selection-{{id}}" data-dependency-value="other" preview-update-target-css="{{target_name}}-{{id}}" preview-update-target-css-property="' . $css_property . '"/>';
			} else {
				$template .= '</select>';
			}
			return $template;
		}
		private static function load_fonts() {
			if (empty(self::$available_fonts)) {
				self::$available_fonts = array(
					'Georgia, serif' => 'Georgia, serif',
					'"Palatino Linotype", "Book Antiqua", Palatino, serif' => '"Palatino Linotype", "Book Antiqua", Palatino, serif',
					'"Times New Roman", Times, serif' => '"Times New Roman", Times, serif',
					'Arial, Helvetica, sans-serif' => 'Arial, Helvetica, sans-serif',
					'"Arial Black", Gadget, sans-serif' => '"Arial Black", Gadget, sans-serif',
					'"Comic Sans MS", cursive, sans-serif' => '"Comic Sans MS", cursive, sans-serif',
					'Impact, Charcoal, sans-serif' => 'Impact, Charcoal, sans-serif',
					'"Lucida Sans Unicode", "Lucida Grande", sans-serif' => '"Lucida Sans Unicode", "Lucida Grande", sans-serif',
					'Tahoma, Geneva, sans-serif' => 'Tahoma, Geneva, sans-serif',
					'"Trebuchet MS", Helvetica, sans-serif' => '"Trebuchet MS", Helvetica, sans-serif',
					'Verdana, Geneva, sans-serif' => 'Verdana, Geneva, sans-serif',
					'"Courier New", Courier, monospace' => '"Courier New", Courier, monospace',
					'"Lucida Console", Monaco, monospace' => '"Lucida Console", Monaco, monospace',
				);
				self::$available_fonts_template = self::generate_dropdown_selection_template(self::$available_fonts, 'font-family');

				self::$available_font_weights = array('100' => '100', '200' => '200', '300' => '300', '400' => 'Normal', '500' => '500', '600' => '600', '700' => 'Bold', '800' => '800', '900' => '900');
				self::$available_font_weights_template = self::generate_dropdown_selection_template(self::$available_font_weights, 'font-weight');

				self::$available_alignments = array(
					'left' => 'Left',
					'center' => 'Center',
					'right' => 'Right'
				);
				self::$available_alignments_template = self::generate_dropdown_selection_template(self::$available_alignments, 'text-align');
			}
		}
		// $mode: 0 - font, 1 - font weight, 2 - alignment
		public static function generate_font_selection_html($base_variable_name, $variable_name, $target_name, $id, $selected, $mode) {
			if (0 === $mode) {
				$template = &self::$available_fonts_template;
				$list = &self::$available_fonts;
			} elseif (1 === $mode) {
				$template = &self::$available_font_weights_template;
				$list = &self::$available_font_weights;
			} else {
				$template = &self::$available_alignments_template;
				$list = &self::$available_alignments;
			}
			$result = str_replace('{{base_variable_name}}', $base_variable_name, $template);
			$result = str_replace('{{variable_name}}', $variable_name, $result);
			$result = str_replace('{{target_name}}', $target_name, $result);
			$result = str_replace('{{id}}', $id, $result);
			$result = str_replace('{{font-value}}', esc_attr($selected), $result);
			$in_list = false;
			foreach($list as $value => $display) {
				$in_list |= $selected === (string)$value;
				$result = str_replace('{{select-' . $value . '}}', $selected === (string)$value ? 'selected' : '', $result);
			}
			$result = str_replace('{{select-other}}', $in_list ? '': 'selected', $result);
			$result = str_replace('{{hide-other}}', $in_list ? 'style="display:none;"': '', $result);
			return $result;
		}
		// </editor-fold>
	}
	require_once(plugin_dir_path(__FILE__) . '/resource/popup-ally-pro-template.php');
	require_once('resource/popupally-pro-widgets.php');
	require_once('resource/utility-functions.php');
	require_once('updater.php');
	PopupAllyPro::init();
}
