<?php
if (!class_exists('PopupAllyProTemplate')) {
	class PopupAllyProTemplate {
		const POPUP_BOX_SHADOW_CSS = '-webkit-box-shadow: 0 10px 25px rgba(0,0,0,0.5);-moz-box-shadow: 0 10px 25px rgba(0,0,0,0.5);box-shadow: 0 10px 25px rgba(0,0,0,0.5);';

		public $uid = null;
		public $template_name = null;
		public $template_order = 0;

		public $frontend_css = null;

		public $display_type_warning = array('timed'=>false, 'exit-intent'=>false, 'scroll'=>false, 'click'=>false, 'embedded'=>false);
		public $style_hide_signup_fields = array('form'=>false, 'name'=>false, 'lname'=>true, 'email'=>false);

		// 0 - normal popup, 1 - embedded, 2 - preview
		protected $popup_html_template_files = null;
		private $popup_html_template = array();

		protected $popup_css_template_files = null;
		private $popup_css_template = array();
		
		public $backend_php = null;
		public $frontend_php = null;
		public $frontend_embedded_php = null;

		private $style_setting_template = null;

		protected $style_template_advanced_customization = array();

		// 0: text, 1: input, 2: submit, 3: responsive text, 4: responsive input, 5: responsive submit
		private static $style_advanced_customization_php = array(
			0 => 'setting-style-text-customization.php',
			1 => 'setting-style-input-customization.php',
			2 => 'setting-style-submit-customization.php',
			3 => 'setting-style-text-responsive-customization.php',
			4 => 'setting-style-input-responsive-customization.php',
			5 => 'setting-style-submit-responsive-customization.php',
			);
		private static $style_advanced_customization_template = array();

		public $html_mapping = null;
		public $no_escape_html_mapping = null;
		public $default_values = null;
	
		public static function initialize(){
			$base_path = dirname(__FILE__) . '/template/';
			foreach(self::$style_advanced_customization_php as $id=>$file) {
				self::$style_advanced_customization_php[$id] = $base_path . $file;
				self::$style_advanced_customization_template[$id] = null;
			}
		}

		protected function __construct() {
		}

		public function sanitize_style($input, $id, $is_active = false) {
			foreach($this->style_template_advanced_customization as $name => $tuple) {
				if ($input[$name . '-font'] === 'other' && isset($input[$name . '-font-other'])) {	// for manually entered font
					$input[$name . '-font'] = $input[$name . '-font-other'];
				}
			}
			if ($is_active) {
				$input['close-iframe-selector'] = '';
				$input['popup-selector'] = '#popup-box-pro-gfcr-' . $id;
				$input['popup-class'] = 'popupally-pro-opened-pro-gfcr-' . $id;
				$input['cookie-name'] = 'popupally-cookie-' . $id;
				$input['close-trigger'] = '.popup-click-close-trigger-' . $id;
			}
			return $input;
		}

		public function prepare_for_code_generation($id, $style, $all_style) {
			return $style;
		}
		public function generate_preview_popup_html($id, $setting) {
			$template = $this->get_popup_html_template(2);
			return $this->generate_popup_html_from_template($id, $setting, $template);
		}
		protected function generate_popup_html_from_template($id, $setting, $template) {
			$template = str_replace('{{num}}', $id, $template);
			foreach ($this->html_mapping as $replace) {
				$template = str_replace('{{' . $replace . '}}', esc_attr($setting[$replace]), $template);
			}
			foreach ($this->no_escape_html_mapping as $replace) {
				$template = str_replace('{{' . $replace . '}}', $setting[$replace], $template);
			}
			// generate hidden fields
			$hidden_fields = '';
			if (isset($setting['hidden-form-fields-name'])) {
				foreach ($setting['hidden-form-fields-name'] as $field_id => $name) {
					$hidden_fields .= '<input type="hidden" name="' . $name . '" value="' . esc_attr($setting['hidden-form-fields-value'][$field_id]) . '"/>';
				}
			}
			if (isset($setting['other-form-fields-name'])) {
				foreach ($setting['other-form-fields-name'] as $field_id => $name) {
					if (isset($setting['used-sign-up-form-fields']) && is_array($setting['used-sign-up-form-fields'])) {
						if (in_array($name, $setting['used-sign-up-form-fields'])) {
							continue;
						}
					}
					$hidden_fields .= '<input type="hidden" name="' . $name . '" value="' . esc_attr($setting['other-form-fields-value'][$field_id]) . '"/>';
				}
			}
			$template = str_replace('{{hidden-fields}}', $hidden_fields, $template);
			return $template;
		}
		public function generate_popup_html($id, $setting, $all_style, $mode = 0) {
			$setting = $this->prepare_for_code_generation($id, $setting, $all_style);
			if (2 == $mode) {
				return $this->generate_preview_popup_html($id, $setting);
			}
			$template = $this->get_popup_html_template($mode);
			return $this->generate_popup_html_from_template($id, $setting, $template);
		}
		public function generate_popup_css($id, $setting, $all_style, $mode) {
			$setting = $this->prepare_for_code_generation($id, $setting, $all_style);
			$template = $this->get_popup_css_template($mode);
			$template = str_replace('{{num}}', $id, $template);

			$show_popup_box_shadow = true;
			foreach ($this->default_values as $replace => $default_value) {
				$value = $setting[$replace];
				if ('-color' === substr($replace, -6)) { // ends with -color
					if ('' === $setting[$replace]) {
						$value = 'transparent';
						if ('background-color' === substr($replace, -16)) { // ends with -background-color
							$show_popup_box_shadow = false;
						}
					} else {
						if ('#' !== $setting[$replace][0]) {
							$value = '#' + $value;
						}
						$value = $value . '000000';
						$value = substr($value, 0, 7);
					}
				}
				$template = str_replace('{{' . $replace . '}}', $value, $template);
			}

			if ($show_popup_box_shadow) {
				$template = str_replace('{{POPUP_BOX_SHADOW_CSS}}', self::POPUP_BOX_SHADOW_CSS, $template);
			} else {
				$template = str_replace('{{POPUP_BOX_SHADOW_CSS}}', '', $template);
			}
			$template = str_replace('{{plugin_uri}}', PopupAllyPro::$PLUGIN_URI, $template);
			return $template;
		}

		protected function get_style_setting_template() {
			if (null === $this->style_setting_template) {
				$this->style_setting_template = file_get_contents($this->backend_php);
			}
			return $this->style_setting_template;
		}

		private static function get_style_advanced_customization_template($template_id) {
			if (null === self::$style_advanced_customization_template[$template_id]) {
				self::$style_advanced_customization_template[$template_id] = file_get_contents(self::$style_advanced_customization_php[$template_id]);
			}
			return self::$style_advanced_customization_template[$template_id];
		}
		// 0: text, 1: input, 2: submit, 3: responsive text, 4: responsive input, 5: responsive submit
		protected static function generate_advanced_customization_code($id, $variable_name, $setting, $name, $template_definition) {
			$template_id = $template_definition[0];
			$target = $target_specific = $template_definition[1];
			if (count($template_definition) > 2) {
				$target_specific = $template_definition[2];
			}
			$advanced_edit = self::get_style_advanced_customization_template($template_id);
			$advanced_edit = str_replace("{{element_name}}", $name, $advanced_edit);
			$advanced_edit = str_replace("{{preview_element_name}}", $target, $advanced_edit);
			$advanced_edit = str_replace("{{preview_element_name_specific}}", $target_specific, $advanced_edit);

			if ($template_id < 3) {
				$select_name = $name . "-font";
				$select_code = PopupAllyPro::generate_font_selection_html($variable_name, $select_name, $target, $id, $setting[$select_name], 0);
				$advanced_edit = str_replace("{{font}}", $select_code, $advanced_edit);

				$select_name = $name . "-font-weight";
				$select_code = PopupAllyPro::generate_font_selection_html($variable_name, $select_name, $target, $id, $setting[$select_name], 1);
				$advanced_edit = str_replace("{{font-weight}}", $select_code, $advanced_edit);

				$select_name = $name . "-align";
				$select_code = PopupAllyPro::generate_font_selection_html($variable_name, $select_name, $target, $id, $setting[$select_name], 2);
				$advanced_edit = str_replace("{{align}}", $select_code, $advanced_edit);
			}
			return $advanced_edit;
		}

		public function show_style_settings($id, $setting) {
			return '';
		}

		public function do_activation_actions(&$new_style, &$lite_style) {
		}

		public function get_popup_html_template($template_id = 0) {
			if (!isset($this->popup_html_template[$template_id])) {
				$this->popup_html_template[$template_id] = file_get_contents($this->popup_html_template_files[$template_id]);
			}
			return $this->popup_html_template[$template_id];
		}

		public function get_popup_css_template($template_id) {
			if (!isset($this->popup_css_template[$template_id])) {
				$this->popup_css_template[$template_id] = file_get_contents($this->popup_css_template_files[$template_id]);
			}
			return $this->popup_css_template[$template_id];
		}

		public function get_popup_dependency(&$style) {
			return false;
		}
		// <editor-fold defaultstate="collapsed" desc="Utility functions">
		public static function hex_to_rgb($hex, $alpha) {
			$hex = str_replace('#', '', $hex);
			$max = strlen($hex) - 1;
			$result = intval(substr($hex, 0, 2), 16) . ', ';
			$result .= intval(substr($hex, min(2, $max), 2), 16) . ', ';
			$result .= intval(substr($hex, min(4, $max), 2), 16) . ', ' . $alpha;
			return $result;
		}

		protected static function save_uploaded_image(&$setting, $image_action, $file_id, $id, $target_param, $dimension_param = false) {
			if('upload' === $setting[$image_action]){
				$setting[$target_param] = '';
				if (isset($_FILES[$file_id . $id]) && !empty($_FILES[$file_id . $id]['size'])) {
					$file_upload_result = wp_handle_upload($_FILES[$file_id . $id], array('test_form' => false));
					if(!empty($file_upload_result['url'])) {
						$image_size_result = @getimagesize($file_upload_result['file']);
						if(false === $image_size_result) {
							add_settings_error('popupally_pro_settings', 'img-upload-error-' . $id, 'Please upload a valid image filetype for Popup #' . $id, 'error');
							if (false !== $dimension_param) {
								$setting[$dimension_param . '-width'] = '0';
								$setting[$dimension_param . '-height'] = '0';
							}
						} else {
							$setting[$target_param] = $file_upload_result['url'];
							if (false !== $dimension_param) {
								$setting[$dimension_param . '-width'] = $image_size_result[0];
								$setting[$dimension_param . '-height'] = $image_size_result[1];
							}
						}
					} else if(!empty($file_upload_result['error'])) {
						add_settings_error('popupally_pro_settings', 'img-upload-error-' . $id, 'File upload error for Popup #' . $id . ': ' . $file_upload_result['error'], 'error');
					}
				}
			}
			unset($setting[$image_action]);
		}
		protected static function image_url_code_generation(&$style, $source_param) {
			if (isset($style[$source_param]) && $style[$source_param]) {
				return 'url(' . $style[$source_param] . ')';
			}
			return 'none';
		}
		protected static function process_field_hidden_status(&$style, $hide_toggle, $html_input_type, $backend_hide_style, $field_name, $source_name){
			if ('true' === $style[$hide_toggle]) {
				$style[$html_input_type] = 'hidden';
				$style[$backend_hide_style] = 'style="display:none;"';
				$style[$field_name] = '';
			} else {
				$style[$html_input_type] = 'text';
				$style[$backend_hide_style] = '';
				$style[$field_name] = $style[$source_name];
				$style['used-sign-up-form-fields'] []= $style[$source_name];
			}
		}
		protected static function replace_image_placeholders($setting, $html, $key, $action_key) {
			if (empty($setting[$key])) {
				$html = str_replace("{{{$action_key}}}", 'upload', $html);
				$html = str_replace("{{{$action_key}-upload-hide}}", 'style="display:none;"', $html);
				$html = str_replace("{{{$action_key}-url-hide}}", '', $html);
			} else {
				$html = str_replace("{{{$action_key}}}", 'url', $html);
				$html = str_replace("{{{$action_key}-upload-hide}}", '', $html);
				$html = str_replace("{{{$action_key}-url-hide}}", 'style="display:none;"', $html);
			}
			return $html;
		}
		protected static function replace_link_type_placeholders($setting, $html, $key) {
			if ('url' === $setting[$key]) {
				$html = str_replace("{{{$key}-url-hide}}", 'style="display:none;"', $html);
				$html = str_replace("{{{$key}-popup-hide}}", '', $html);
			} else {
				$html = str_replace("{{{$key}-url-hide}}", '', $html);
				$html = str_replace("{{{$key}-popup-hide}}", 'style="display:none;"', $html);
			}
			return $html;
		}
		// </editor-fold>
	}
	PopupAllyProTemplate::initialize();
}