<?php
if (!class_exists('PopupAllyProImageTemplate')) {
	class PopupAllyProImageTemplate extends PopupAllyProTemplate {
		public function __construct() {
			parent::__construct();
			$this->uid = 'lquydg';
			$this->template_name = 'Just Image';
			$this->template_order = 10;

			$this->backend_php = dirname(__FILE__) . '/backend/image-pro-preview.php';
			$this->display_type_warning = array('timed'=>true, 'exit-intent'=>true, 'scroll'=>true, 'click'=>false, 'embedded'=>true);
			$this->style_hide_signup_fields = array('form'=>true, 'name'=>true, 'lname'=>true, 'email'=>true);

			// 0: front end html, 1: front end embedded, 2: backend preview
			$this->popup_html_template_files = array(
				0 => dirname(__FILE__) . '/frontend/image-pro-popup.php',
				1 => dirname(__FILE__) . '/frontend/image-pro-embedded.php',
				2 => dirname(__FILE__) . '/backend/image-pro-preview-template.php',
				3 => dirname(__FILE__) . '/backend/image-pro-960-preview-template.php',
				4 => dirname(__FILE__) . '/backend/image-pro-640-preview-template.php',
			);
			// 0: front end; 1: backend, 2: front end top margin
			$this->popup_css_template_files = array(
				0 => dirname(__FILE__) . '/frontend/image-pro-popup.css',
				1 => dirname(__FILE__) . '/backend/image-pro-preview-popup.css',
				2 => dirname(__FILE__) . '/frontend/image-pro-popup-top-margin.css',
			);

			$this->html_mapping = array('lquydg-overlay-close-trigger');
			$this->no_escape_html_mapping = array();
			$this->default_values = array(
				'lquydg-background-color' => '#45ae7a',
				'lquydg-background-image-url' => '',
				'lquydg-background-image' => 'none',
				'lquydg-width' => '840',
				'lquydg-height' => '540',
				'lquydg-half-width' => '420',
				'lquydg-half-height' => '270',
				'lquydg-image-1' => 'none',
				'lquydg-image-1-url' => PopupAllyPro::$PLUGIN_URI . 'resource/img/screenshot.png',
				'lquydg-image-1-width' => '800',
				'lquydg-image-1-height' => '500',
				'lquydg-image-1-margin-top' => '20',
				'lquydg-image-1-margin-bottom' => '20',
				'lquydg-overlay-color' => "#505050",
				'lquydg-overlay-opacity' => '0.5',
				'lquydg-overlay-color-rgba' => '80,80,80,0.5',
				'lquydg-disable-overlay-close' => 'false',

				'lquydg-width-960' => '640',
				'lquydg-height-960' => '415',
				'lquydg-half-width-960' => '320',
				'lquydg-half-height-960' => '207',
				'lquydg-image-1-960-width' => '600',
				'lquydg-image-1-960-height' => '375',
				'lquydg-image-1-960-margin-top' => '20',
				'lquydg-image-1-960-margin-bottom' => '20',

				'lquydg-width-640' => '440',
				'lquydg-height-640' => '290',
				'lquydg-half-width-640' => '220',
				'lquydg-half-height-640' => '145',
				'lquydg-image-1-640-width' => '400',
				'lquydg-image-1-640-height' => '250',
				'lquydg-image-1-640-margin-top' => '20',
				'lquydg-image-1-640-margin-bottom' => '20',
			);
			$this->style_template_advanced_customization = array(
			);
			$this->style_template_checked_replace = array(
				'lquydg-disable-overlay-close',
			);
		}

		public function do_activation_actions(&$new_style, &$lite_style) {
		}

		public function sanitize_style($setting, $id, $is_active = false) {
			$setting = parent::sanitize_style($setting, $id, $is_active);
			PopupAllyProTemplate::save_uploaded_image($setting, 'lquydg-background-image-action', 'lquydg-background-image-file-', $id, 'lquydg-background-image-url');
			PopupAllyProTemplate::save_uploaded_image($setting, 'lquydg-image-1-action', 'lquydg-image-1-file-', $id, 'lquydg-image-1-url', 'lquydg-image-1');
			return $setting;
		}

		public function prepare_for_code_generation($id, $style, $all_style) {
			$style['lquydg-background-image'] = PopupAllyProTemplate::image_url_code_generation($style, 'lquydg-background-image-url');
			$style['lquydg-image-1'] = PopupAllyProTemplate::image_url_code_generation($style, 'lquydg-image-1-url');

			$style['lquydg-half-width'] = intval($style['lquydg-width']) / 2;
			$style['lquydg-half-height'] = intval($style['lquydg-height']) / 2;
			$style['lquydg-half-width-960'] = intval($style['lquydg-width-960']) / 2;
			$style['lquydg-half-height-960'] = intval($style['lquydg-height-960']) / 2;
			$style['lquydg-half-width-640'] = intval($style['lquydg-width-640']) / 2;
			$style['lquydg-half-height-640'] = intval($style['lquydg-height-640']) / 2;

			// disable background close trigger
			if ('true' === $style['lquydg-disable-overlay-close']) {
				$style['lquydg-overlay-close-trigger'] = '';
			} else {
				$style['lquydg-overlay-close-trigger'] = ' popup-click-close-trigger-' . $id;
			}

			$style['lquydg-overlay-color-rgba'] = PopupAllyProTemplate::hex_to_rgb($style['lquydg-overlay-color'], $style['lquydg-overlay-opacity']);
			return $style;
		}
		public function generate_preview_popup_html($id, $setting) {
			$generated_templates = array();
			for ($i=2;$i<=4;++$i) {
				$template = $this->get_popup_html_template($i);
				$generated_templates[$i] = $this->generate_popup_html_from_template($id, $setting, $template);
			}
			return $generated_templates;
		}
		public function show_style_settings($id, $setting) {
			$code = PopupAllyPro::get_popup_code();
			$preview_code = $code[$id]['preview_html'][$this->uid];
			$html = $this->get_style_setting_template();
			$variable_name = '[' . $id . ']';

			foreach($this->style_template_advanced_customization as $name => $tuple) {
				$advanced_edit = PopupAllyProTemplate::generate_advanced_customization_code($id, $variable_name, $setting, $name, $tuple);

				$html = str_replace("{{{$name}-advanced}}", $advanced_edit, $html);
			}

			$html = str_replace('{{id}}', $id, $html);
			for ($i=2;$i<=4;++$i) {
				$html = str_replace("{{preview-code-$i}}", $preview_code[$i], $html);
			}
			$html = str_replace('{{variable_name}}', $variable_name, $html);

			$html = PopupAllyProTemplate::replace_image_placeholders($setting, $html, 'lquydg-background-image-url', 'lquydg-background-image-action');
			$html = PopupAllyProTemplate::replace_image_placeholders($setting, $html, 'lquydg-image-1-url', 'lquydg-image-1-action');

			// the order is important, as style_template_checked_replace is a subset of default_values
			foreach($this->style_template_checked_replace as $param) {
				$html = str_replace("{{{$param}}}", 'true' === $setting[$param] ? 'checked' : '', $html);
			}
			foreach($this->default_values as $param  => $default_value) {
				$html = str_replace("{{{$param}}}", esc_attr($setting[$param]), $html);
			}
			return $html;
		}
	}
	PopupAllyPro::add_template(new PopupAllyProImageTemplate());
}
