<?php

if (!class_exists('PopupAllyProUpdater')) {
	class PopupAllyProUpdater {
		const PRODUCT_URL = '';
		const UPDATE_SLUG = 'popupally-pro';
		const UPDATE_URL = '';
		const LATEST_UPDATE_CACHE = 'popupally_pro_latest_update';
		const PLUGIN_ERROR_MESSAGE_CACHE = 'popupally_pro_error_message';
		const PLUGIN_ERROR_MESSAGE_PREFIX = 'PopupAlly Pro Error: ';
		private static $plugin_slug = null;
		private static $admin_messages = array();
		private static $current_version = null;

		public static function init() {
			self::add_actions();
			self::add_filters();
			self::$plugin_slug = basename(dirname(__FILE__)) . '/popup-ally-pro.php';
			self::$current_version = PopupAllyPro::VERSION;
		}

		private static function get_license_info() {
			return PopupAllyPro::get_license_settings();
		}

		private static function process_serial_status($is_valid) {
			PopupAllyPro::set_enable_setting($is_valid);
		}

		public static function do_activation_actions() {
			delete_site_transient(self::LATEST_UPDATE_CACHE);
			delete_site_transient(self::PLUGIN_ERROR_MESSAGE_CACHE);
		}

		public static function do_deactivation_actions() {
			delete_site_transient(self::LATEST_UPDATE_CACHE);
			delete_site_transient(self::PLUGIN_ERROR_MESSAGE_CACHE);
		}

		private static function add_actions() {
			if (is_admin()) {
				add_action('admin_init', array(__CLASS__, 'auto_check_update_message'));
				add_action('admin_notices', array(__CLASS__, 'show_admin_message'));
				add_action('admin_notices', array(__CLASS__, 'show_error_admin_message'));
			}

			// schedule update version check
			add_action('popupally_pro_check_update_event', array(&$this, 'check_update_periodically'));
			if (!wp_next_scheduled('popupally_pro_check_update_event')) {
				wp_schedule_event(current_time('timestamp'), 'twicedaily', 'popupally_pro_check_update_event');
			}
		}

		public static function add_filters() {
			// auto update
			add_filter('plugins_api', array(__CLASS__, 'get_plugin_info'), 10, 3);
			add_filter('pre_set_site_transient_update_plugins', array(__CLASS__, 'force_check_update'));
			add_filter('transient_update_plugins', array(__CLASS__, 'check_update'));
			add_filter('site_transient_update_plugins', array(__CLASS__, 'check_update'));
		}

		// <editor-fold defaultstate="collapsed" desc="Auto update">
		public static function force_check_update($transient = false)
		{
			if (false !== $transient && (isset($transient->response[self::$plugin_slug]) || !property_exists($transient, 'checked'))) {
				return $transient;
			}
			$update_info = self::get_plugin_update(true);
			if (!$update_info) {
				return $transient;
			}
			$remote_version = $update_info['version'];
			if (version_compare(self::$current_version, $remote_version, '<')) {
				$license = PopupAllyPro::get_license_settings();
				$obj = new stdClass();
				$obj->slug = self::UPDATE_SLUG;
				$obj->new_version = $remote_version;
				$obj->url = self::PRODUCT_URL;
				$obj->package = add_query_arg(array('email' => $license['email'], 'serial' => $license['serial'], 'action' => 'download'), self::UPDATE_URL);
				$obj->plugin = self::$plugin_slug;
				$transient->response[self::$plugin_slug] = $obj;
			}
			return $transient;
		}

		public static function check_update($transient)
		{
			$update_info = self::get_plugin_update();
			if (!$update_info) {
				return $transient;
			}
			$remote_version = $update_info['version'];
			if (version_compare(self::$current_version, $remote_version, '<')) {
				$license = PopupAllyPro::get_license_settings();
				$obj = new stdClass();
				$obj->slug = self::UPDATE_SLUG;
				$obj->new_version = $remote_version;
				$obj->url = self::PRODUCT_URL;
				$obj->package = add_query_arg(array('email' => $license['email'], 'serial' => $license['serial'], 'action' => 'download'), self::UPDATE_URL);
				$obj->plugin = self::$plugin_slug;
				$transient->response[self::$plugin_slug] = $obj;
			}
			return $transient;
		}
		public static function check_update_periodically() {
			if (defined('WP_INSTALLING')) return false;
			self::get_plugin_update(true);
		}
		public static function auto_check_update_message() {
			$update_info = self::get_plugin_update();
			if ($update_info === false) {
				return;
			}
			if (is_array($update_info)) {
				$messages = $update_info['message'];
				if (is_array($messages)) {
					foreach ($messages as $message) {
						self::$admin_messages []= array(false, $message);
					}
				}
				$remote_version = $update_info['version'];
				if (version_compare(self::$current_version, $remote_version, '<')) {
					if (is_multisite()) {
						$upgrade_url = admin_url() . 'network/update-core.php';
					} else {
						$upgrade_url = admin_url() . 'update-core.php';
					}
					$message = 'PopupAlly Pro ' . $remote_version . ' is available! <a href="' . $upgrade_url . '">Please update now</a>.';
					self::$admin_messages []= array(false, $message);
				}
			}
		}

		public static function show_admin_message() {
			foreach (self::$admin_messages as $message) {
				echo '<div class="update-nag ' . ($message[0]?'error':'update') . '">';
				echo $message[1] . '</div>';
			}
		}

		public static function show_error_admin_message() {
			$messages = get_site_transient(self::PLUGIN_ERROR_MESSAGE_CACHE);
			if (is_array($messages)) {
				foreach ($messages as $message) {
					echo '<div class="error"><p>' . self::PLUGIN_ERROR_MESSAGE_PREFIX . $message . '</p></div>';
				}
			}
		}

		public static function get_plugin_update($force = false, $license = false) {
			$update_info = get_site_transient(self::LATEST_UPDATE_CACHE);
			if (false === $update_info || $force) {
				$update_info = self::get_remote_information('update', $license);
				if ($update_info === false) {
					set_site_transient(self::LATEST_UPDATE_CACHE, 0, 60 * 60 * 12);
					return false;
				}
				set_site_transient(self::LATEST_UPDATE_CACHE, $update_info, 60 * 60 * 12);
			}
			if (0 === $update_info) {
				return false;
			}
			return $update_info;
		}
		public static function get_plugin_info($false, $action, $arg)
		{
			if ($arg->slug === self::UPDATE_SLUG) {
				$information = self::get_remote_information('info');
				if (false === $information) {
					return false;
				}
				return $information['info'];
			}
			return false;
		}
		public static function get_remote_information($action, $license = false) {
			if (false === $license) {
				$license = self::get_license_info();
			}
			
			self::process_serial_status(true);
			delete_site_transient(self::PLUGIN_ERROR_MESSAGE_CACHE);
			return false;
		}
		// </editor-fold>
	}
	PopupAllyProUpdater::init();
}